@@ -15,7 +15,7 @@ use Carp;
 use List::Util qw(uniq);
 use POSIX qw(strftime);
 
-our $VERSION = '0.059'; # VERSION
+our $VERSION = '0.060'; # VERSION
 
 use Sub::Quote;
 
@@ -105,9 +105,10 @@ has 'bag_encoding' => (
 );
 
 has 'bag_info' => (
-    is      => 'rw',
-    lazy    => 1,
-    builder => '_build_bag_info',
+    is        => 'rw',
+    lazy      => 1,
+    builder   => '_build_bag_info',
+    predicate => 1
 );
 
 has 'errors' => (
@@ -187,17 +188,7 @@ has 'tagmanifest_files' => (
     builder => '_build_tagmanifest_files',
 );
 
-has 'manifest_entries' => (
-    is => 'ro',
-    lazy => 1,
-    builder => '_build_manifest_entries',
-);
 
-has 'tagmanifest_entries' => (
-    is => 'ro',
-    lazy => 1,
-    builder => '_build_tagmanifest_entries',
-);
 
 has 'payload_files' => ( # relatively to bagit base
     is => 'ro',
@@ -217,7 +208,9 @@ has 'plugins' => (
 );
 
 has 'manifests' => (
-    is=>'rw',
+    is      => 'rw',
+    lazy    => 1,
+    builder => '_build_manifests'
     #isa=>'HashRef',
 );
 
@@ -311,47 +304,6 @@ sub _build_tagmanifest_files {
   return \@tagmanifest_files;
 }
 
-sub __build_xxxmanifest_entries {
-  my ($self, $xxmanifestfiles) = @_;
-  my @xxmanifests = @{$xxmanifestfiles};
-  my $xxmanifest_entries = {};
-  my $bag_path=$self->bag_path();
-  foreach my $xxmanifest_file (@xxmanifests) {
-    die("Cannot open $xxmanifest_file: $!") unless (open(my $XXMANIFEST,"<:encoding(utf8)", $xxmanifest_file));
-    my $algo = $xxmanifest_file;
-    $algo =~ s#//#/#g; # to fix problems with double path-separators
-    $algo =~ s#^($bag_path/).bagit/#$1#; # FIXME: only for dotbagit-variant, if dotbagit will be outdated, this should be removed
-    $algo =~ s#^$bag_path/##;
-    $algo =~ s#tag(manifest-[a-z0-9]+\.txt)#$1#;
-    $algo =~ s#.*manifest-([a-z0-9]+)\.txt$#$1#;
-    if ($algo =~ m#/#) {
-        die "wrong replacement of path $xxmanifest_file to determine algorithm '$algo' correctly, please contact author";
-    }
-    while (my $line = <$XXMANIFEST>) {
-      chomp($line);
-      my($digest,$file) = split(/\s+/, $line, 2);
-      next unless ((defined $digest) && (defined $file)); # empty lines!
-      $xxmanifest_entries->{$algo}->{$file} = $digest;
-    }
-    close($XXMANIFEST);
-  }
-  return $xxmanifest_entries;
-}
-
-sub _build_tagmanifest_entries {
-  my ($self) = @_;
-  my @tm_files = $self->tagmanifest_files();
-  my $entries = $self->__build_xxxmanifest_entries(@tm_files);
-  return $entries;
-}
-
-sub _build_manifest_entries {
-  my ($self) = @_;
-  my @m_files = $self->manifest_files();
-  my $entries = $self->__build_xxxmanifest_entries(@m_files);
-  return $entries;
-}
-
 sub _build_payload_files{
   my($self) = @_;
   my $payload_dir = $self->payload_path;
@@ -373,6 +325,7 @@ sub _build_payload_files{
     }
     #print "name: ".$File::Find::name."\n";
   }, $payload_dir);
+  @payload = sort @ payload;
   return wantarray ? @payload : \@payload;
 
 }
@@ -484,15 +437,18 @@ sub _build_bag_info {
     my ($self) = @_;
     my $bagit = $self->metadata_path;
     my $file = join("/", $bagit, "bag-info.txt");
-    open(my $BAGINFO, "<", $file) or die("Cannot read $file: $!");
-    my @lines;
-    foreach my $line (<$BAGINFO>) {
-        push @lines, $line;
+    if (-e $file) {
+        open(my $BAGINFO, "<", $file) or di("Cannot read $file: $!");
+        my @lines;
+        foreach my $line (<$BAGINFO>) {
+            push @lines, $line;
+        }
+        close($BAGINFO);
+        my $lines = join("", @lines);
+        return $self->_parse_bag_info($lines);
     }
-    close($BAGINFO);
-    my $lines = join("", @lines);
-    return $self->_parse_bag_info ($lines);
-
+    # bag-info.txt is optional
+    return;
 }
 
 sub _build_non_payload_files {
@@ -517,7 +473,7 @@ sub _build_non_payload_files {
     }
     #print "name: ".$File::Find::name."\n";
   }, $self->metadata_path);
-
+  @non_payload = sort @non_payload;
   return wantarray ? @non_payload : \@non_payload;
 
 }
@@ -549,187 +505,7 @@ sub load_plugins {
     return 1;
 }
 
-sub __calc_digest {
-    my ($digestobj, $filename) = @_;
-    open(my $fh, '<', $filename) || die("Can't open '$filename', $!");
-    binmode($fh);
-    my $digest = $digestobj->get_hash_string($fh);
-    close $fh || die("could not close file '$filename', $!");
-    return $digest;
-}
 
-# calc digest
-# expects digestobj, expected_ref, array_ref of filenames
-# returns arrayref of hashes where each entry has
-# $tmp->{calculated_digest} = $digest;
-# $tmp->{expected_digest} = $expected_digest;
-# $tmp->{filename} = $filename;
-sub calc_digests {
-    my ($self, $bagit, $digestobj, $filenames_ref) = @_;
-    my @digest_hashes;
-    # check if we could use parallel
-    my $is_parallelizeable;
-    if (($self->has_parallel()) && (defined $self->parallel)) {
-        my $err;
-        ($is_parallelizeable, $err) = Class::Load::try_load_class("Parallel::Iterator");
-        if (!$is_parallelizeable) {
-            warn "Class 'Parallel::Iterator' could not be loaded…, $err\n";
-            $self->{parallel} = undef;
-        }
-    }
-    if ($is_parallelizeable) {
-        my $class = Class::Load::load_class("Parallel::Iterator");
-        $class->import( qw(iterate_as_array));
-        @digest_hashes = iterate_as_array(
-            sub {
-                my ($idx, $localname) = @_;
-                my $fullname = $bagit ."/". $localname;
-                my $tmp;
-                $tmp->{calculated_digest} = __calc_digest($digestobj, $fullname);
-                $tmp->{local_name} = $localname;
-                $tmp->{full_name} = $fullname;
-                $tmp;
-            }, $filenames_ref);
-    }
-    else { # fallback to serial processing
-        @digest_hashes = map {
-            my $localname = $_;
-            my $fullname = $bagit ."/". $localname;
-            my $tmp;
-            $tmp->{calculated_digest} = __calc_digest($digestobj, $fullname);
-            $tmp->{local_name} = $localname;
-            $tmp->{full_name} = $fullname;
-            $tmp;
-        } @{$filenames_ref};
-    }
-    return \@digest_hashes;
-}
-
-sub check_payload_filepath_conformity{
-    my $local_name = shift;
-    # HINT: there is no guarantuee *not* to escape!
-    return
-    ($local_name =~ m/^~/) # Unix Home
-        || ($local_name =~ m#\./#) # Unix, parent dir escape
-        || ($local_name =~ m#^[A-Z]:[\\/]#) # Windows Drive
-        || ($local_name =~ m#^/#) # Unix absolute path
-        || ($local_name =~ m#^$#) # Unix Env
-        || ($local_name =~ m#^\\#) # Windows absolute path
-        || ($local_name =~ m#^%[^%]*%#) # Windows ENV
-        || ($local_name =~ m#^\*#) # Artifact of md5sum-Tool, where ' *' is allowed to separate checksum and file in fixity line
-    ;
-}
-
-sub _verify_XXX_manifests {
-    my ($self, $xxprefix, $xxmanifest_entries, $files, $return_all_errors) =@_;
-    # Read the manifest file
-    my @payload = @{ $files };
-    my @invalid_messages;
-    my $bagit = $self->bag_path;
-    my $version = $self->bag_version();
-    my $subref_invalid_report_or_die = sub {
-        my $message = shift;
-        if (defined $return_all_errors) {
-            push @invalid_messages, $message;
-        } else {
-            die($message);
-        }
-        return;
-    };
-    foreach my $local_name (@payload) {
-        # local_name is relative to bagit base
-        unless (-r $bagit."/".$local_name) {
-            &$subref_invalid_report_or_die(
-                "cannot read $local_name (bag-path:$bagit)",
-            );
-        }
-    }
-    # Evaluate each file against the manifest
-    foreach my $alg (keys %{$xxmanifest_entries}) {
-        my $manifest_alg = $self->manifests->{$alg};
-        next unless (defined $manifest_alg); # FIXME_ errormessage?
-        my $digestobj = $manifest_alg->algorithm();
-        my $local_xxfilename = "${xxprefix}-${alg}.txt";
-        my $xxfilename = "${bagit}$local_xxfilename";
-
-        # first check if each file from payload exists in manifest_entries for given alg
-        foreach my $local_name (@payload) {
-            # local_name is relative to bagit base
-            unless (exists $xxmanifest_entries->{$alg}->{$local_name}) { # localname as value should exist!
-                &$subref_invalid_report_or_die(
-                    "file '$local_name' found, which is not in '$local_xxfilename' (bag-path:'$bagit')!",
-                );
-            }
-        }
-        # second check if each file from manifest_entries for given alg exists in payload
-        foreach my $local_name (keys %{$xxmanifest_entries->{$alg}}) {
-            if ( # to avoid escapes via manifest-files
-                check_payload_filepath_conformity( $local_name )
-            ) {
-                &$subref_invalid_report_or_die("file '$local_name' not allowed in '$local_xxfilename' (bag-path:'$bagit'")
-            }
-            else {
-                unless (List::Util::any {$_ eq $local_name} @payload) {
-                    &$subref_invalid_report_or_die(
-                        "file '$local_name' NOT found, but expected via '$local_xxfilename' (bag-path:'$bagit')!"
-                    );
-                }
-            }
-        }
-        # all preconditions full filled, now calc all digests
-        my $digest_hashes_ref = $self->calc_digests($bagit, $digestobj, \@payload);
-        # compare digests
-        if (defined $digest_hashes_ref && (ref $digest_hashes_ref eq 'ARRAY')) {
-            foreach my $digest_entry (@{$digest_hashes_ref}) {
-                $digest_entry->{expected_digest} = $xxmanifest_entries->{$alg}->{
-                    $digest_entry->{local_name}
-                };
-                if ($digest_entry->{calculated_digest} ne $digest_entry->{expected_digest}) {
-                    &$subref_invalid_report_or_die(
-                        sprintf("file '%s' invalid, digest (%s) calculated=%s, but expected=%s in file '%s'",
-                            $digest_entry->{local_name},
-                            $alg,
-                            $digest_entry->{calculated_digest},
-                            $digest_entry->{expected_digest},
-                            $xxfilename
-                        )
-                    );
-                }
-            }
-        }
-    }
-    if($return_all_errors && (scalar @invalid_messages > 0)) {
-        push @{ $self->{errors} },
-            join("\n\t",
-                "bag verify for bagit version '$version' failed with invalid files:",
-                sort @invalid_messages
-            );
-        return;
-    }
-    return 1;
-}
-
-sub _verify_manifests {
-    my ($self, $alg, $return_all_errors) = @_;
-    return $self->_verify_XXX_manifests(
-        "manifest",
-        $self->manifest_entries(),
-        $self->payload_files(),
-        $return_all_errors
-    );
-}
-
-sub _verify_tagmanifests {
-    my ($self, $alg, $return_all_errors) = @_;
-    # filter tagmanifest-files
-    my @non_payload_files = grep { $_ !~ m/tagmanifest-[a-z0-9]+\.txt/} @{ $self->non_payload_files };
-    return $self->_verify_XXX_manifests(
-        "tagmanifest",
-        $self->tagmanifest_entries(),
-        \@non_payload_files,
-        $return_all_errors
-    );
-}
 
 
 
@@ -747,7 +523,6 @@ sub verify_bag {
     my $payload_dir   = $self->payload_path;
     my $return_all_errors = $opts->{return_all_errors};
 
-
     if (-f $fetch_file) {
         die("Fetching via file '$fetch_file' is not supported by current Archive::BagIt implementation")
     }
@@ -760,19 +535,25 @@ sub verify_bag {
 
     # check forced fixity
 
-    my $ret_manifests = $self->_verify_manifests($forced_fixity_alg, $return_all_errors);
-    my $ret_tagmanifests = $self->_verify_tagmanifests($forced_fixity_alg, $return_all_errors);
-    if ($ret_manifests && $ret_tagmanifests) {
-        return 1;
-    } else {
-        die join("\n", @{ $self->errors });
+    my @errors;
+    foreach my $algorithm ( keys %{ $self->manifests }) {
+        my $res = $self->manifests->{$algorithm}->verify_manifest($self->payload_files, $return_all_errors);
+        if (defined $res && $res ne 1) { push @errors, $res; }
+    }
+    foreach my $algorithm ( keys %{ $self->manifests }) {
+        my $res = $self->manifests->{$algorithm}->verify_tagmanifest($self->non_payload_files, $return_all_errors);
+        if (defined $res && $res ne 1) { push @errors, $res; }
+    }
+    push @{$self->{errors}}, @errors;
+    my $err = $self->errors();
+    my @err =  @{ $err };
+    if (scalar( @err ) > 0) {
+        die join("\n","bag verify for bagit version '$version' failed with invalid files.", @err);
     }
     return 1;
 }
 
 
-
-
 sub calc_payload_oxum {
     my($self) = @_;
     my @payload = @{$self->payload_files};
@@ -797,6 +578,7 @@ sub calc_bagsize {
     else { return sprintf "%0.2f TB", $octets/(1024*1024*1024*1024); }
 }
 
+
 sub create_bagit {
     my($self) = @_;
     my $metadata_path = $self->metadata_path();
@@ -806,6 +588,7 @@ sub create_bagit {
     return 1;
 }
 
+
 sub create_baginfo {
     my($self) = @_; # because bag-info.txt allows multiple key-value-entries, hash is replaced
     $self->_add_or_replace_bag_info('Bagging-Date', POSIX::strftime("%F", gmtime(time)));
@@ -875,7 +658,6 @@ sub init_metadata {
 }
 
 
-
 sub make_bag {
   my ($class, $bag_path) = @_;
   my $isa = ref $class;
@@ -903,23 +685,32 @@ Archive::BagIt::Base
 
 =head1 VERSION
 
-version 0.059
+version 0.060
 
 =head1 SYNOPSIS
 
 This modules will hopefully help with the basic commands needed to create
 and verify a bag. This part supports BagIt 1.0 according to RFC 8493 ([https://tools.ietf.org/html/rfc8493](https://tools.ietf.org/html/rfc8493)).
 
+You only need to know the following methods first:
+
+=head2 read a BagIt
+
     use Archive::BagIt::Base;
 
     #read in an existing bag:
     my $bag_dir = "/path/to/bag";
     my $bag = Archive::BagIt::Base->new($bag_dir);
 
+=head2 construct a BagIt around a payload
 
-    #construct bag in an existing directory
+    use Archive::BagIt::Base;
     my $bag2 = Archive::BagIt::Base->make_bag($bag_dir);
 
+=head2 verify a BagIt-dir
+
+    use Archive::BagIt::Base;
+
     # Validate a BagIt archive against its manifest
     my $bag3 = Archive::BagIt::Base->new($bag_dir);
     my $is_valid1 = $bag3->verify_bag();
@@ -934,7 +725,7 @@ Archive::BagIt::Base
 
 =head1 VERSION
 
-version 0.059
+version 0.060
 
 =head1 NAME
 
@@ -1017,6 +808,8 @@ See L<https://datatracker.ietf.org/doc/rfc8493/?include_text=1> for details.
 
 =back
 
+=head1 METHODS
+
 =head2 Constructor
 
 The constructor sub, will create a bag with a single argument,
@@ -1055,11 +848,12 @@ As default SHA512 and MD5 will be loaded and therefore used. If you want to crea
 checksum-algorithm, you could use this method to (re-)register it. It expects list of strings with namespace of type:
 Archive::BagIt::Plugin::Algorithm::XXX where XXX is your chosen fixity algorithm.
 
-=head2 verify_bag
+=head2 verify_bag($opts)
 
-An interface to verify a bag.
+A method to verify a bag deeply. If C<$opts> is set with C<{return_all_errors}> all fixity errors are reported.
+The default ist to die with error message if any error is detected.
 
-You might also want to check Archive::BagIt::Fast to see a more direct way of accessing files (and thus faster).
+HINT: You might also want to check Archive::BagIt::Fast to see a more direct way of accessing files (and thus faster).
 
 =head2 calc_payload_oxum()
 
@@ -1069,17 +863,25 @@ returns an array with octets and streamcount of payload-dir
 
 returns a string with human readable size of paylod
 
-=head2 store
+=head2 create_bagit()
 
-store a bagit-obj if bagit directory-structure was already constructed,
+creates a bagit.txt file
 
-=head2 init_metadata
+=head2 create_baginfo()
+
+creates a bag-info.tx
+
+=head2 store()
+
+store a bagit-obj if bagit directory-structure was already constructed.
+
+=head2 init_metadata()
 
 A constructor that will just create the metadata directory
 
 This won't make a bag, but it will create the conditions to do that eventually
 
-=head2 make_bag
+=head2 make_bag( $bag_path )
 
 A constructor that will make and return a bag from a directory,
 
@@ -1108,7 +910,7 @@ Rob Schmidt <rjeschmi@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2020 by Rob Schmidt and William Wueppelmann.
+This software is copyright (c) 2020 by Rob Schmidt and William Wueppelmann and Andreas Romeyke.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
@@ -1119,7 +921,7 @@ Rob Schmidt <rjeschmi@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2020 by Rob Schmidt and William Wueppelmann.
+This software is copyright (c) 2020 by Rob Schmidt and William Wueppelmann and Andreas Romeyke.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
