@@ -39,6 +39,9 @@ sub get_hash_string {
     my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,
         $atime,$mtime,$ctime,$blksize,$blocks)
         = stat $fh;
+    if ((!defined $blksize) || (length($blksize) < 1) || ($blksize <1)) {
+        $blksize = 8192;
+    }
     my $buffer;
     while (read($fh, $buffer, $blksize)) {
         $self->_digest->add($buffer);
@@ -70,7 +73,7 @@ Archive::BagIt::Plugin::Algorithm::MD5 - The default MD5 algorithm plugin
 
 =head1 VERSION
 
-version 0.059
+version 0.060
 
 =head1 NAME
 
@@ -78,7 +81,7 @@ Archive::BagIt::Plugin::Algorithm::MD5 - The default MD5 algorithm plugin
 
 =head1 VERSION
 
-version 0.059
+version 0.060
 
 =head1 AVAILABILITY
 
@@ -102,7 +105,7 @@ Rob Schmidt <rjeschmi@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2020 by Rob Schmidt and William Wueppelmann.
+This software is copyright (c) 2020 by Rob Schmidt and William Wueppelmann and Andreas Romeyke.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
@@ -113,7 +116,7 @@ Rob Schmidt <rjeschmi@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2020 by Rob Schmidt and William Wueppelmann.
+This software is copyright (c) 2020 by Rob Schmidt and William Wueppelmann and Andreas Romeyke.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
